<?php
/**
 *
 * SugarCRM Community Edition is a customer relationship management program developed by
 * SugarCRM, Inc. Copyright (C) 2004-2013 SugarCRM Inc.
 *
 * SuiteCRM is an extension to SugarCRM Community Edition developed by SalesAgility Ltd.
 * Copyright (C) 2011 - 2020 SalesAgility Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by the
 * Free Software Foundation with the addition of the following permission added
 * to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK
 * IN WHICH THE COPYRIGHT IS OWNED BY SUGARCRM, SUGARCRM DISCLAIMS THE WARRANTY
 * OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License along with
 * this program; if not, see http://www.gnu.org/licenses or write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 *
 * You can contact SugarCRM, Inc. headquarters at 10050 North Wolfe Road,
 * SW2-130, Cupertino, CA 95014, USA. or at email address contact@sugarcrm.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License version 3,
 * these Appropriate Legal Notices must retain the display of the "Powered by
 * SugarCRM" logo and "Supercharged by SuiteCRM" logo. If the display of the logos is not
 * reasonably feasible for technical reasons, the Appropriate Legal Notices must
 * display the words "Powered by SugarCRM" and "Supercharged by SuiteCRM".
 */

if (!defined('sugarEntry') || !sugarEntry) {
    die('Not A Valid Entry Point');

}

require_once('include/database/DBManagerFactory.php');

global $sugar_version;

/**
 * upgrade wizard logging
 * @param $entry
 */
function _logThis($entry)
{
    if (function_exists('logThis')) {
        logThis($entry);
    } else {

        $log = clean_path(getcwd() . '/upgradeWizard.log');
        // create if not exists
        if (!file_exists($log)) {
            $fp = fopen($log, 'wb+'); // attempts to create file
            if (!is_resource($fp)) {
                $GLOBALS['log']->fatal('UpgradeWizard could not create the upgradeWizard.log file');
            }
        } else {
            $fp = fopen($log, 'ab+'); // write pointer at end of file
            if (!is_resource($fp)) {
                $GLOBALS['log']->fatal('UpgradeWizard could not open/lock upgradeWizard.log file');
            }
        }

        $line = date('r') . ' [UpgradeWizard] - ' . $entry . "\n";

        if (fwrite($fp, $line) === false) {
            $GLOBALS['log']->fatal('UpgradeWizard could not write to upgradeWizard.log: ' . $entry);
        }

        fclose($fp);
    }
}

/**
 * This is specific for MSSQL. Before doing an alter table statement for MSSQL, this function will drop all the constraint
 * for that column
 * @param $tableName
 * @param $columnName
 */
function dropColumnConstraintForMSSQL($tableName, $columnName)
{
    global $sugar_config;
    if ($sugar_config['dbconfig']['db_type'] === 'mssql') {
        $db = DBManagerFactory::getInstance();
        $query = 'declare @name nvarchar(32), @sql nvarchar(1000)';

        $query .= " select @name = sys.objects.name from sys.objects where type_desc like '%CONSTRAINT' and (OBJECT_NAME(parent_object_id) like '%{$tableName}%') and sys.objects.object_id in (select default_object_id from sys.columns where name like '{$columnName}')";

        $query .= " begin
		    select @sql = 'ALTER TABLE {$tableName} DROP CONSTRAINT [' + @name + ']'
		    execute sp_executesql @sql
		end";

        $db->query($query);
    }
}

/**
 * gets Upgrade version
 */
function getUpgradeVersion()
{
    $version = '';

    if (isset($_SESSION['sugar_version_file']) && !empty($_SESSION['sugar_version_file']) && is_file($_SESSION['sugar_version_file'])) {
        // do an include because the variables will load locally, and it will only populate in here.
        include($_SESSION['sugar_version_file']);

        return $sugar_db_version;
    }

    return $version;
}

// moving rebuild js to upgrade utils

function rebuild_js_lang()
{
    require_once('include/utils/file_utils.php');
    global $sugar_config;

    $jsFiles = array();
    getFiles($jsFiles, $sugar_config['cache_dir'] . 'jsLanguage');
    foreach ($jsFiles as $file) {
        unlink($file);
    }

    if (empty($sugar_config['js_lang_version'])) {
        $sugar_config['js_lang_version'] = 1;
    } else {
        ++$sugar_config['js_lang_version'];
    }

    write_array_to_file('sugar_config', $sugar_config, 'config.php');

    //remove language cache files
    require_once('include/SugarObjects/LanguageManager.php');
    LanguageManager::clearLanguageCache();
}

function clear_SugarLogic_cache()
{
    require_once('include/utils/file_utils.php');
    global $sugar_config;

    $files = array();
    getFiles($files, $sugar_config['cache_dir'] . 'Expressions');
    foreach ($files as $file) {
        unlink($file);
    }
}


/**
 * update DB version and sugar_version.php
 * @param $version
 */

function upgradeDbAndFileVersion($version)
{
    global $instancePath;
    if (!isset($instancePath) && isset($_SESSION['instancePath'])) {
        $instancePath = $_SESSION['instancePath'];
    }
    if (!function_exists('updateVersions')) {
        if (file_exists('modules/UpgradeWizard/uw_utils.php')) {
            require_once('modules/UpgradeWizard/uw_utils.php');
        } elseif (file_exists($instancePath . '/modules/UpgradeWizard/uw_utils.php')) {
            require_once($instancePath . '/modules/UpgradeWizard/uw_utils.php');
        }
    }
    updateVersions($version);
}

function rebuild_dashlets()
{
    if (is_file('cache/dashlets/dashlets.php')) {
        unlink('cache/dashlets/dashlets.php');
    }

    require_once('include/Dashlets/DashletCacheBuilder.php');
    $dc = new DashletCacheBuilder();
    $dc->buildCache();
}

function rebuild_roles()
{
    $_REQUEST['upgradeWizard'] = true;
    require_once('data/SugarBean.php');
    global $ACLActions, $beanList, $beanFiles;
    include('modules/ACLActions/actiondefs.php');
    include('include/modules.php');
    include('modules/ACL/install_actions.php');
}

function upgrade_LDAP()
{
    require_once('modules/Administration/Administration.php');
    $focus = new Administration();
    $focus->retrieveSettings('ldap', true);
    if (isset($focus->settings['ldap_admin_user']) && !empty($focus->settings['ldap_admin_user'])) {
        $focus->saveSetting('ldap', 'authentication', '1');
    } elseif (isset($focus->settings['ldap_admin_user'])) {
        $focus->saveSetting('ldap', 'authentication', '0');
    }

}

/**
 * @param string $minVersion
 * @param string $operator
 * @throws Exception
 */
function checkPHP($minVersion = '5.5.0', $operator = '<=')
{
    if (version_compare(PHP_VERSION, $minVersion, $operator)) {
        $errMessage = 'Invalid PHP version, acceptable PHP version is ' . $minVersion . '+. ';
        echo $errMessage . '<br>
            <form method="post" action="">

                <input type="hidden" name="module" value="UpgradeWizard">
                <input type="hidden" name="action" value="index">
                <input type="hidden" name="step" value="-1">
                <input type="submit" class="button" value="Cancel">
            </form>';
        throw new Exception($errMessage);
        //exit;
    }

}

/**
 * @param $origVersion
 * @param $destVersion
 * @param $queryType
 * @param string $resumeFromQuery
 */
function runSqlFiles($origVersion, $destVersion, $queryType, $resumeFromQuery = '')
{
    global $sugar_config, $_SESSION;

    // This flag is determined by the preflight check in the installer
    if (!isset($_SESSION['schema_change']) || $_SESSION['schema_change'] === 'sugar') {

        _logThis("Upgrading the database from {$origVersion} to version {$destVersion}");

        $origVersion = implodeVersion($origVersion, 3, 'x');
        $destVersion = implodeVersion($destVersion, 3, 'x');

        $schemaFileName = $origVersion . '_to_' . $destVersion;

        switch ($sugar_config['dbconfig']['db_type']) {
            case 'mysql':
                $schemaFileName .= '_mysql.sql';
                break;
            case 'mssql':
                $schemaFileName .= '_mssql.sql';
                break;
        }

        $schemaFile = $_SESSION['unzip_dir'] . '/scripts/' . $schemaFileName;
        _logThis("Running SQL file $schemaFile");

        if (is_file($schemaFile)) {
            ob_start();
            @parseAndExecuteSqlFile($schemaFile, $queryType, $resumeFromQuery);
            ob_end_clean();
        } elseif (strcmp($origVersion, $destVersion) == 0) {
            _logThis('*** Skipping schema upgrade for point release.');
        } else {
            _logThis("*** ERROR: Schema change script [{$schemaFile}] could not be found!");
        }

    } else {
        _logThis('*** Skipping Schema Change Scripts - Admin opted to run queries manually and should have done so by now.');
    }
}

function genericFunctions()
{
    //REBUILD JS LANG
    _logThis('Rebuilding JS Langauages');
    rebuild_js_lang();

    //REBUILD DASHLETS
    _logThis('Rebuilding Dashlets');
    rebuild_dashlets();
}

